#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @File    :   ezkfg\__init__.py
# @Time    :   2022-05-13 21:30:11
# @Author  :   Bingjie Yan
# @Email   :   bj.yan.pa@qq.com
# @License :   Apache License 2.0


from .config import *
from .handler import *

__version__ = "0.0.1a3"
